@@ -1,6 +1,5 @@
 /* testutils.c */
 
-
 #include "testutils.h"
 
 #include "cbc.h"
@@ -14,7 +13,7 @@
 #include <unistd.h>
 
 /* -1 means invalid */
-const signed char hex_digits[0x100] =
+static const signed char hex_digits[0x100] =
   {
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
@@ -108,6 +107,19 @@ decode_hex_dup(const char *hex)
     }
 }
 
+void
+print_hex(unsigned length, uint8_t *data)
+{
+  unsigned i;
+  
+  for (i = 0; i < length; i++)
+    {
+      if (! (i % 8))
+        printf(" ");
+      printf("%02x", data[i]);
+    }
+}
+
 int verbose = 0;
 
 int
@@ -228,11 +240,12 @@ test_armor(const struct nettle_armor *armor,
            const uint8_t *data,
            const uint8_t *ascii)
 {
+  void *ctx = alloca(armor->context_size);
   uint8_t *buffer = alloca(1 + strlen(ascii));
   uint8_t *check = alloca(1 + data_length);
 
-  memset(buffer, 0, 1 + strlen(ascii));
-  memset(check, 0, 1 + data_length);
+  memset(buffer, 0x33, 1 + strlen(ascii));
+  memset(check, 0x55, 1 + data_length);
 
   if (strlen(ascii) != armor->encode(buffer, data_length, data))
     FAIL();
@@ -240,31 +253,21 @@ test_armor(const struct nettle_armor *armor,
   if (!MEMEQ(strlen(ascii), buffer, ascii))
     FAIL();
 
-  if (buffer[strlen(ascii)])
+  if (0x33 != buffer[strlen(ascii)])
     FAIL();  
 
-  if (data_length != armor->decode(check, strlen(ascii), buffer))
+  armor->decode_init(ctx);
+  if (data_length != armor->decode_update(ctx, check, strlen(ascii), buffer))
     FAIL();
 
   if (!MEMEQ(data_length, check, data))
     FAIL();
 
-  if (check[data_length])
+  if (0x55 != check[data_length])
     FAIL();
 }
 
 #if HAVE_LIBGMP
-#define SIGN(key, hash, msg, signature) do {	\
-  hash##_update(&hash, LDATA(msg));		\
-  rsa_##hash##_sign(key, &hash, signature);	\
-} while(0)
-
-#define VERIFY(key, hash, msg, signature) (	\
-  hash##_update(&hash, LDATA(msg)),		\
-  rsa_##hash##_verify(key, &hash, signature)	\
-)
-
-
 /* Missing in current gmp */
 static void
 mpz_togglebit (mpz_t x, unsigned long int bit)
@@ -274,6 +277,18 @@ mpz_togglebit (mpz_t x, unsigned long int bit)
   else
     mpz_setbit(x, bit);
 }
+#endif /* HAVE_LIBGMP */
+
+#if WITH_PUBLIC_KEY
+#define SIGN(key, hash, msg, signature) do {	\
+  hash##_update(&hash, LDATA(msg));		\
+  rsa_##hash##_sign(key, &hash, signature);	\
+} while(0)
+
+#define VERIFY(key, hash, msg, signature) (	\
+  hash##_update(&hash, LDATA(msg)),		\
+  rsa_##hash##_verify(key, &hash, signature)	\
+)
 
 void
 test_rsa_md5(struct rsa_public_key *pub,
@@ -360,4 +375,76 @@ test_rsa_sha1(struct rsa_public_key *pub,
 
   mpz_clear(signature);
 }
-#endif /* HAVE_LIBGMP */
+
+void
+test_rsa_key(struct rsa_public_key *pub,
+	     struct rsa_private_key *key)
+{
+  mpz_t tmp;
+  mpz_t phi;
+  
+  mpz_init(tmp); mpz_init(phi);
+  
+  if (verbose)
+    {
+      /* FIXME: Use gmp_printf */
+      fprintf(stderr, "Public key: n=");
+      mpz_out_str(stderr, 16, pub->n);
+      fprintf(stderr, "\n    e=");
+      mpz_out_str(stderr, 16, pub->e);
+
+      fprintf(stderr, "\n\nPrivate key: d=");
+      mpz_out_str(stderr, 16, key->d);
+      fprintf(stderr, "\n    p=");
+      mpz_out_str(stderr, 16, key->p);
+      fprintf(stderr, "\n    q=");
+      mpz_out_str(stderr, 16, key->q);
+      fprintf(stderr, "\n    a=");
+      mpz_out_str(stderr, 16, key->a);
+      fprintf(stderr, "\n    b=");
+      mpz_out_str(stderr, 16, key->b);
+      fprintf(stderr, "\n    c=");
+      mpz_out_str(stderr, 16, key->c);
+      fprintf(stderr, "\n\n");
+    }
+
+  /* Check n = p q */
+  mpz_mul(tmp, key->p, key->q);
+  if (mpz_cmp(tmp, pub->n))
+    FAIL();
+
+  /* Check c q = 1 mod p */
+  mpz_mul(tmp, key->c, key->q);
+  mpz_fdiv_r(tmp, tmp, key->p);
+  if (mpz_cmp_ui(tmp, 1))
+    FAIL();
+
+  /* Check ed = 1 (mod phi) */
+  mpz_sub_ui(phi, key->p, 1);
+  mpz_sub_ui(tmp, key->q, 1);
+
+  mpz_mul(phi, phi, tmp);
+
+  mpz_mul(tmp, pub->e, key->d);
+  mpz_fdiv_r(tmp, tmp, phi);
+  if (mpz_cmp_ui(tmp, 1))
+    FAIL();
+
+  /* Check a e = 1 (mod (p-1) ) */
+  mpz_sub_ui(phi, key->p, 1);
+  mpz_mul(tmp, pub->e, key->a);
+  mpz_fdiv_r(tmp, tmp, phi);
+  if (mpz_cmp_ui(tmp, 1))
+    FAIL();
+  
+  /* Check b e = 1 (mod (q-1) ) */
+  mpz_sub_ui(phi, key->q, 1);
+  mpz_mul(tmp, pub->e, key->b);
+  mpz_fdiv_r(tmp, tmp, phi);
+  if (mpz_cmp_ui(tmp, 1))
+    FAIL();
+  
+  mpz_clear(tmp); mpz_clear(phi);
+}
+#endif /* WITH_PUBLIC_KEY */
+
