@@ -12,6 +12,10 @@
 #include <stdio.h>
 
 #if HAVE_LIBGMP
+# include "bignum.h"
+#endif
+
+#if WITH_PUBLIC_KEY
 # include "rsa.h"
 #endif
 
@@ -29,6 +33,9 @@ decode_hex(uint8_t *dst, const char *hex);
 const uint8_t *
 decode_hex_dup(const char *hex);
 
+void
+print_hex(unsigned length, uint8_t *data);
+
 /* The main program */
 int
 test_main(void);
@@ -64,7 +71,7 @@ test_armor(const struct nettle_armor *armor,
            const uint8_t *data,
            const uint8_t *ascii);
 
-#if HAVE_LIBGMP
+#if WITH_PUBLIC_KEY
 void
 test_rsa_md5(struct rsa_public_key *pub,
 	     struct rsa_private_key *key,
@@ -74,7 +81,12 @@ void
 test_rsa_sha1(struct rsa_public_key *pub,
 	      struct rsa_private_key *key,
 	      mpz_t expected);
-#endif /* HAVE_LIBGMP */
+
+void
+test_rsa_key(struct rsa_public_key *pub,
+	     struct rsa_private_key *key);
+
+#endif /* WITH_PUBLIC_KEY */
 
 #define H2(d, s) decode_hex((d), (s))
 #define H(x) decode_hex_dup(x)
@@ -83,9 +95,14 @@ test_rsa_sha1(struct rsa_public_key *pub,
 #define LDATA(x) strlen(x), x
 
 #define MEMEQ(length, a, b) (!memcmp((a), (b), (length)))
+#define MEMEQH(length, a, b) \
+((length) == decode_hex_length((b)) \
+ && !memcmp((a), decode_hex_dup((b)), (length)))
 
 #define FAIL() abort()
 #define SKIP() exit(77)
 #define SUCCESS() return EXIT_SUCCESS
 
+#define ASSERT(x) do { if (!(x)) FAIL(); } while(0)
+
 #endif /* NETTLE_TESTUTILS_H_INCLUDED */
